@@ -103,8 +103,8 @@ class DeepSpeedStrategy(DDPStrategy):
         offload_params_device: str = "cpu",
         nvme_path: str = "/local_nvme",
         params_buffer_count: int = 5,
-        params_buffer_size: int = 1e8,
-        max_in_cpu: int = 1e9,
+        params_buffer_size: int = 100_000_000,
+        max_in_cpu: int = 1_000_000_000,
         offload_optimizer_device: str = "cpu",
         optimizer_buffer_count: int = 4,
         block_size: int = 1048576,
@@ -113,13 +113,13 @@ class DeepSpeedStrategy(DDPStrategy):
         overlap_events: bool = True,
         thread_count: int = 1,
         pin_memory: bool = False,
-        sub_group_size: int = 1e12,
+        sub_group_size: int = 1_000_000_000_000,
         contiguous_gradients: bool = True,
         overlap_comm: bool = True,
         allgather_partitions: bool = True,
         reduce_scatter: bool = True,
-        allgather_bucket_size: int = 2e8,
-        reduce_bucket_size: int = 2e8,
+        allgather_bucket_size: int = 200_000_000,
+        reduce_bucket_size: int = 200_000_000,
         zero_allow_untested_optimizer: bool = True,
         logging_batch_size_per_gpu: Union[str, int] = "auto",
         config: Optional[Union[Path, str, dict]] = None,
@@ -757,6 +757,9 @@ class DeepSpeedStrategy(DDPStrategy):
             TypeError:
                 If ``storage_options`` arg is passed in
         """
+        # broadcast the filepath from rank 0 to ensure all the states are saved in a common filepath
+        filepath = self.broadcast(filepath)
+
         if storage_options is not None:
             raise TypeError(
                 "`Trainer.save_checkpoint(..., storage_options=...)` with `storage_options` arg"
