@@ -14,10 +14,14 @@
 from __future__ import annotations
 
 import sys
+import threading
 from types import ModuleType, TracebackType
 
 import pytorch_lightning.utilities.argparse
 
+# Create a global lock to ensure no race condition with deleting sys modules
+_lock = threading.Lock()
+
 
 class pl_legacy_patch:
     """Registers legacy artifacts (classes, methods, etc.) that were removed but still need to be included for
@@ -35,6 +39,7 @@ class pl_legacy_patch:
     """
 
     def __enter__(self) -> None:
+        _lock.acquire()
         # `pl.utilities.argparse_utils` was renamed to `pl.utilities.argparse`
         legacy_argparse_module = ModuleType("pytorch_lightning.utilities.argparse_utils")
         sys.modules["pytorch_lightning.utilities.argparse_utils"] = legacy_argparse_module
@@ -49,3 +54,4 @@ class pl_legacy_patch:
         if hasattr(pytorch_lightning.utilities.argparse, "_gpus_arg_default"):
             delattr(pytorch_lightning.utilities.argparse, "_gpus_arg_default")
         del sys.modules["pytorch_lightning.utilities.argparse_utils"]
+        _lock.release()
